Imagine you know nothing about the construction of physical things (imagine) and you've just moved into an empty apartment. You hire an "interior engineer" to set the place up for you. You have many requirements but for the purpose of this example we'll just focus on two: 1) A fish tank - you like fish; and 2) A liquor cabinet - you like booze.

You arrive home from work and the place is set up and ready to go. The engineer has done a great job. You thank him and he tells you if you have any new requirements or need any modifications to just give him a call. 

A couple of weeks later you decide you don't really like the liquor cabinet. You want a different one. Something shorter and longer so all the bottles form a long line.

---

Dr. Alan Kay says stuff like this

> Until real software engineering is developed, the next best practice is to develop with a dynamic system that has extreme late binding in all aspects.

source: ?

and this

> It’s not that people are completely stupid, but if there’s a big idea and you have deadlines and you have expedience and you have competitors, very likely what you’ll do is take a low-pass filter on that idea and implement one part of it and miss what has to be done next. This happens over and over again. If you’re using early-binding languages as most people do, rather than late-binding languages, then you really start getting locked in to stuff that you’ve already done. You can’t reformulate things that easily.

source: http://queue.acm.org/detail.cfm?id=1039523

and this

> I thought of objects being like biological cells and/or individual computers on a network, only able to communicate with messages

> The subsequent Smalltalk's ... backslid towards Simula and did not replace the extension mechanisms with safer ones that were anywhere near as useful.

> OOP to me means only messaging, local retention and protection and hiding of state-process, and extreme late-binding of all things.

source: <http://userpage.fu-berlin.de/~ram/pub/pub_jf47ht81Ht/doc_kay_oop_en>

In that email conversation Alan talks a lot about hiding and "getting rid of" data. My understanding is that what he means by "data" is the raw values in memory and the references that point to them - such as "variables" in most modern programming languages.

Take this line of code

    city = users.perform('get city', 'bob')

If the `perform` method turns it's arguments into a formatted message, sends that message somewhere, waits for a response, then returns that response it can be seen as "dataless" because the return value is, at best, a *copy* of the value of some memory region at the place where the message was received. The program that sent the message has no idea what data structures are used by the program that received the message and responded to it. Even if the program responds with a message formatted as CSV (or JSON, or XML) it's not "data" in this sense of the word until it's loaded into memory and something holds a reference to it.

Of course the *fastest* way to implement that line is to use "data". It's both faster to write - you don't need to bother message creation, transmission, reception, plus dealing with any errors that might occur - and faster to run because the procedure just has to look something up in local memory.

---

For all this talk about messaging and late binding. Alan Kay didn't implement it with smalltalk 80 (but did perhaps with earlier versions).

His pals over at ARPA did though. So that's the place to look for a messaging architecture. The internet.

So we have to look elsewhere for a message architecture.

---

Actors & Encapsulation. Encapsulation is phase 1. Req/rep over sockets. Actors is phase 2, unidirectional messages, pub/sub, reliability.

---

Sockets are great. They allow two or more programs to communicate without requiring those programs have access to the same regions of memory.

Netcat (`nc`) is a command-line utility for doing simple things with sockets. Here we create a socket bound to port 2000 and "listen" for incoming data. Then in another terminal we "connect" to that socket. Now if you type anything in either terminal and press return those characters will be sent over the socket and printed in the other terminal.

      Terminal 1                          Terminal 2
    ------------------------------------ ------------------------------------
    $ nc -l 2000                        | $ nc localhost 2000
    hello                               | hello


The socket API is less friendly. Sockets are very low level. They're unix's mechanism for sending raw streams of bytes from one machine to another. So it's up to the application using them to set up a loop that asks the operating system if it's received any new connections on a socket and if it's received any new bytes from currently open connections. A fair amount of code needs to be written just to manage that data. Framing, buffering, error handling, it all ends up being a ton of work. 

Developers often decide to just adapt and existing system (usually HTTP libraries) that abstracts over all of that stuff even if the model isn't a perfect fit for their application and provides more features (and requires more boilerplate and massaging) than they really need.

Message sockets are lower-level than HTTP but higher level than raw posix sockets.

There are two major differences between message sockets and raw sockets. First all communication over message sockets is done with discreet messages (hence the name). You don't send a stream of bytes, you send a message, and on the other end you never receive a stream of bytes (or "data" event), you either get the whole message or you don't.

The second difference is that message sockets send and receive messages in one of 3 patterns: push/pull, req/rep, pub/sub which means each socket instance is one of 6 types: push, pull, req, rep, pub, sub.

The easiest way to see how these work is to see them in action. 

Let's start with push/pull. 